Ext.namespace("dSS", "dSS.config", "dSS.devprops");
// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dsid
dSS.devprops.ShadePropertiesForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        if (config.dsid === undefined) {
            Ext.MessageBox.alert(dSS._("Error"), ("Missing dSID"));
            return;
        }

        this.dsid = config.dsid;

        this.addEvents('formsubmitted', 'formloaded');

        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded' ],
            padding: 5,
            height: 185,
            scope: this,
            trackResetOnLoad: true,
            layout: 'absolute',
            items:
            [
                {
                    xtype: 'label',
                    text: dSS._("Motion time open-close") + ':',
                    x: 6,
                    y: 13
                },
                {
                    xtype: 'numberfield',
                    ref: 'openclose',
                    name: 'openclose',
                    allowBlank: false,
                    allowDecimals: false,
                    allowNegative: false,
                    maxValue: 655,
                    width: 160,
                    x: 275,
                    y: 10
                },
                {
                    xtype: 'label',
                    text: dSS._("s"),
                    x: 440,
                    y: 13
                },
                {
                    xtype: 'label',
                    text: dSS._("Motion time close-open") + ':',
                    x: 6,
                    y: 48
                },
                {
                    xtype: 'numberfield',
                    ref: 'closeopen',
                    name: 'closeopen',
                    allowBlank: false,
                    allowDecimals: false,
                    allowNegative: false,
                    maxValue: 655,
                    width: 160,
                    x: 275,
                    y: 45
                },
                {
                    xtype: 'label',
                    text: dSS._("s"),
                    x: 440,
                    y: 48
                },
                {
                    xtype: 'label',
                    text: dSS._("Motion time fins") + ':',
                    x: 6,
                    y: 83
                },
                {
                    xtype: 'numberfield',
                    ref: 'fins',
                    name: 'fins',
                    allowBlank: false,
                    allowDecimals: false,
                    allowNegative: false,
                    maxValue: 655,
                    width: 160,
                    x: 275,
                    y: 80
                },
                {
                    xtype: 'label',
                    text: dSS._("s"),
                    x: 440,
                    y: 83
                },
                {
                    xtype: 'label',
                    text: dSS._("Delay time") + ':',
                    x: 6,
                    y: 118
                },
                {
                    xtype: 'numberfield',
                    ref: 'delaytime',
                    name: 'delaytime',
                    allowBlank: false,
                    allowDecimals: false,
                    allowNegative: false,
                    maxValue: 2550,
                    width: 160,
                    x: 275,
                    y: 115
                },
                {
                    xtype: 'label',
                    text: dSS._("ms"),
                    x: 440,
                    y: 118
                },
                {
                    xtype: 'label',
                    text: dSS._("Direction of travel") + ':',
                    x: 6,
                    y: 153
                },
                {
                    xtype: 'combo',
                    ref: 'direction',
                    name: 'direction',
                    typeAhead: false,
                    lazyRender: true,
                    mode: 'local',
                    forceSelection: true,
                    editable: false,
                    triggerAction: 'all',
                    store: new Ext.data.ArrayStore({
                      id: 0,
                        fields:
                        [
                            'id',
                            'displayText'
                        ],
                        data:
                        [
                            [0, dSS._("Default")],
                            [1, dSS._("Reverse")]
                        ]
                    }),
                    valueField: 'id',
                    displayField: 'displayText',
                    width: 160,
                    x: 275,
                    y: 150
                }
            ]
        });

        this.firstLoad = true;

        this.loaded = {
            open_close: false,
            close_open: false,
            motion_time_fins: false,
            delay_time: false,
            direction_of_travel: false
        };

        dSS.devprops.ShadePropertiesForm.superclass.constructor.apply(this, arguments);

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }

                var val;
                if (data.json.ok === true) {
                    val = data.json.result.value;
                }

                switch (data.loader) {
                    case dSS.devprops.Loaders.SHADE_OPEN_CLOSE:
                        this.loaded.open_close = true;
                        if (data.json.ok === true) {
                            this.openclose.setValue(Math.floor(val*10/1000));
                            dSS.util.ResetDirtyField(this.openclose);

                        } else {
                            this.openclose.markInvalid();
                        }
                        break;
                    case dSS.devprops.Loaders.SHADE_CLOSE_OPEN:
                        this.loaded.close_open = true;
                        if (data.json.ok === true) {
                            this.closeopen.setValue(Math.floor(val*10/1000));
                            dSS.util.ResetDirtyField(this.closeopen);
                        } else {
                            this.closeopen.markInvalid();
                        }
                        break;
                    case dSS.devprops.Loaders.SHADE_MTIME_FINS:
                        this.loaded.motion_time_fins = true;
                        if (data.json.ok === true) {
                            this.fins.setValue(Math.floor(val*10/1000));
                            dSS.util.ResetDirtyField(this.fins);
                        } else {
                            this.fins.markInvalid();
                        }
                        break;
                    case dSS.devprops.Loaders.SHADE_DELAY_TIME:
                        this.loaded.delay_time = true;
                        if (data.json.ok === true) {
                            this.delaytime.setValue(val * 10);
                            dSS.util.ResetDirtyField(this.delaytime);
                        } else {
                            this.delaytime.markInvalid();
                        }
                        break;
                    case dSS.devprops.Loaders.SHADE_DIR_TRAVEL_RTIME_CH:
                        this.loaded.direction_of_travel = true;
                        if (data.json.ok === true) {
                            this.directionValue = (val & 0x0000ff00) >> 8;
                            // 1-bit
                            if ((this.directionValue & 1) > 0) {
                                this.direction.setValue(1);
                            } else {
                                this.direction.setValue(0);
                            }
                        } else {
                            this.direction.markInvalid();
                        }
                        break;
                    default:
                        return;
                }

                if ((this.loaded.open_close === true) &&
                    (this.loaded.close_open === true) &&
                    (this.loaded.motion_time_fins === true) &&
                    (this.loaded.delay_time === true) &&
                    (this.loaded.direction_of_travel === true)) {
                    this.fireEvent('formloaded', true);
                }

            },
            this);

        this.on('beforedestroy', function() {
                this.closed = true;
            },
            this);

        this.on('submit', function() {
                if (this.getForm().isValid() !== true) {
                    Ext.Msg.alert(dSS._("Error"),
                                  dSS._("Please correct values in the red " +
                                        "marked fields and try again"));
                    return;
                }

                var save = [];

                if (this.openclose.isDirty()) {
                    var val = Math.floor(this.openclose.getValue()*1000/10);
                    var dec = dSS.util.decode16(val);
                    save.push({
                        url: '/json/device/setConfig',
                        scope: this,
                        params:
                        {
                            'class': 3,
                            index: 0x32,
                            dsid: this.dsid,
                            value: dec.low
                        },
                        failureMsg: dSS._("Could not save motion time " +
                                          "open-close settings")
                    });

                    save.push({
                        url: '/json/device/setConfig',
                        scope: this,
                        params:
                        {
                            'class': 3,
                            index: 0x33,
                            dsid: this.dsid,
                            value: dec.high
                        },
                        failureMsg: dSS._("Could not save motion time " +
                                          "open-close settings")
                    });
                }

                if (this.closeopen.isDirty()) {
                    var val = Math.floor(this.closeopen.getValue()*1000/10);
                    var dec = dSS.util.decode16(val);
                    save.push({
                        url: '/json/device/setConfig',
                        scope: this,
                        params:
                        {
                            'class': 3,
                            index: 0x30,
                            dsid: this.dsid,
                            value: dec.low
                        },
                        failureMsg: dSS._("Could not save motion time " +
                                          "close-open settings")
                    });

                    save.push({
                        url: '/json/device/setConfig',
                        scope: this,
                        params:
                        {
                            'class': 3,
                            index: 0x31,
                            dsid: this.dsid,
                            value: dec.high
                        },
                        failureMsg: dSS._("Could not save motion time " +
                                          "close-open settings")
                    });
                }

                if (this.fins.isDirty()) {
                    var val = Math.floor(this.fins.getValue()*1000/10);
                    var dec = dSS.util.decode16(val);
                    save.push({
                        url: '/json/device/setConfig',
                        scope: this,
                        params:
                        {
                            'class': 3,
                            index: 0x3e,
                            dsid: this.dsid,
                            value: dec.low
                        },
                        failureMsg: dSS._("Could not save motion time " +
                                          "fins settings")
                    });

                    save.push({
                        url: '/json/device/setConfig',
                        scope: this,
                        params:
                        {
                            'class': 3,
                            index: 0x3f,
                            dsid: this.dsid,
                            value: dec.high
                        },
                        failureMsg: dSS._("Could not save motion time " +
                                          "fins settings")
                    });

                }

                if (this.delaytime.isDirty()) {
                    save.push({
                        url: '/json/device/setConfig',
                        scope: this,
                        params:
                        {
                            'class': 3,
                            index: 0x34,
                            dsid: this.dsid,
                            value: Math.floor(this.delaytime.getValue() / 10)
                        },
                        failureMsg: dSS._("Could not save delay time settings")
                    });
                }

                var d = this.directionValue;

                if (this.direction.getValue() === 0) {
                    d = d & ~1;
                } else {
                    d = d | 1;
                }

                if (this.directionValue != d) {
                    save.push({
                        url: '/json/device/setConfig',
                        scope: this,
                        params:
                        {
                            'class': 3,
                            index: 0x37,
                            dsid: this.dsid,
                            value: d
                        },
                        failureMsg: dSS._("Could not save direction of " +
                                          "travel settings")
                    });
                }

                this.saveOK = true;
                var call = this.saveData.createDelegate(this, [ save, true ]);
                call();
            },
            this);

    },
    saveDataCallback: function(records, state) {
        var call = this.saveData.createDelegate(this, [ records, state ]);
        call();
    },
    saveData: function(records, ok) {
        if (ok === false) {
            this.saveOK = false;
        }
        if (records.length < 1) {
            if (this.saveOK === false) {
                dSS.MessageBus.fireEvent('statusmessage',
                    dSS._("Error") + ": " +
                    dSS._("Some properties could not be saved"));
            }
            this.fireEvent('formsubmitted');
            return;
        }

        var record = records.shift();

        var cb = function(state) {
            var call = this.saveDataCallback.createDelegate(this,
                                                            [ records, state ]);
            call();
        };
        dSS.util.genericRequest(record, cb.createDelegate(this));
    }
});

Ext.reg('shadepropsform', dSS.devprops.ShadePropertiesForm);

// add advanced form, make sure it can communicate with the standard form

dSS.devprops.ShadePropertiesAdvancedForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        if (config.dsid === undefined) {
            Ext.MessageBox.alert(dSS._("Error"), ("Missing dSID"));
            return;
        }

        this.dsid = config.dsid;

        this.addEvents('formsubmitted', 'formloaded');

        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded' ],
            padding: 5,
            height: 218,
            scope: this,
            border: false,
            layout: 'absolute',
            items:
            [
                {
                    xtype: 'label',
                    text: dSS._("Motion when blinking") + ':',
                    x: 6,
                    y: 13
                },
                {
                    xtype: 'numberfield',
                    ref: 'mblink',
                    allowBlank: false,
                    allowDecimals: false,
                    allowNegative: false,
                    maxValue: 8415, // 0xff * 33ms
                    x: 275,
                    y: 10
                },
                {
                    xtype: 'label',
                    text: dSS._("ms"),
                    x: 440,
                    y: 13
                },
                {
                    xtype: 'label',
                    text: dSS._("Reverse time on direction change") + ':',
                    x: 6,
                    y: 48
                },
                {
                    xtype: 'numberfield',
                    ref: 'reverse',
                    allowBlank: false,
                    allowDecimals: false,
                    allowNegative: false,
                    maxValue: 2550,
                    x: 275,
                    y: 45
                },
                {
                    xtype: 'label',
                    text: dSS._("ms"),
                    x: 440,
                    y: 48
                },
                {
                    xtype: 'label',
                    text: dSS._("Follow-up time at end position") + ':',
                    x: 6,
                    y: 83
                },
                {
                    xtype: 'numberfield',
                    ref: 'followup',
                    allowBlank: false,
                    allowDecimals: false,
                    allowNegative: false,
                    maxValue: 25500, // ms, 0xff * 0.1s
                    x: 275,
                    y: 80
                },
                {
                    xtype: 'label',
                    text: dSS._("ms"),
                    x: 440,
                    y: 83
                },
                {
                    xtype: 'label',
                    text: dSS._("Threshold of end switch recognition") + ':',
                    x: 6,
                    y: 118
                },
                {
                    xtype: 'numberfield',
                    ref: 'threshold',
                    allowBlank: false,
                    allowDecimals: false,
                    allowNegative: false,
                    maxValue: 14790, // 0xff * 58
                    x: 275,
                    y: 115
                },
                {
                    xtype: 'label',
                    text: dSS._("Number of steps for fine tuning adjustment") + ':',
                    x: 6,
                    y: 153
                },
                {
                    xtype: 'numberfield',
                    ref: 'numsteps',
                    allowBlank: false,
                    allowDecimals: false,
                    allowNegative: false,
                    maxValue: 0xff,
                    x: 275,
                    y: 150
                },
                {
                    xtype: 'label',
                    text: dSS._("Motion time for fine tuning adjustment") + ':',
                    x: 6,
                    y: 188
                },
                {
                    xtype: 'numberfield',
                    ref: 'mtimeadjust',
                    allowBlank: false,
                    allowDecimals: false,
                    allowNegative: false,
                    maxValue: 2550,
                    x: 275,
                    y: 185
                },
                {
                    xtype: 'label',
                    text: dSS._("ms"),
                    x: 440,
                    y: 188
                }
            ]
        });

        this.loaded = {
            motion_time_blink: false,
            reverse_time_on_dirchange: false,
            ftime_endpos_threshold_endsw: false
        };

        dSS.devprops.ShadePropertiesAdvancedForm.superclass.constructor.apply(this, arguments);

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }

                var val;
                if (data.json.ok === true) {
                    val = data.json.result.value;
                }

                switch (data.loader) {
                    case dSS.devprops.Loaders.SHADE_MTIME_BLINK:
                        this.loaded.motion_time_blink = true;
                        if (data.json.ok === true) {
                            this.mblink.setValue(val * 33);
                            dSS.util.ResetDirtyField(this.mblink);
                        } else {
                            this.mblink.markInvalid();
                        }
                        break;
                    case dSS.devprops.Loaders.SHADE_DIR_TRAVEL_RTIME_CH:
                        this.loaded.reverse_time_on_dirchange = true;
                        if (data.json.ok === true) {
                            var rt = (val & 0x000000ff);
                            this.reverse.setValue(rt * 10);
                            dSS.util.ResetDirtyField(this.reverse);
                        } else {
                            this.reverse.markInvalid();
                        }
                        break;
                    case dSS.devprops.Loaders.SHADE_FTEND_TRESHOLDENDSW:
                        this.loaded.ftime_endpos_threshold_endsw = true;
                        if (data.json.ok === true) {
                            var follow_up = val & 0x000000ff;
                            var threshold = (val & 0x0000ff00) >> 8;
                            this.followup.setValue(follow_up * 100);
                            dSS.util.ResetDirtyField(this.followup);
                            this.threshold.setValue(threshold * 58);
                            dSS.util.ResetDirtyField(this.threshold);
                        } else {
                            this.followup.markInvalid();
                            this.threshold.markInvalid();
                        }
                        break;
                    case dSS.devprops.Loaders.SHADE_NUMST_MTIME_FTUNE:
                        this.loaded.num_steps_mtime_finetune = true;
                        if (data.json.ok === true) {
                            var steps = val & 0x000000ff;
                            var mtime = (val & 0x0000ff00) >> 8;
                            this.numsteps.setValue(steps);
                            dSS.util.ResetDirtyField(this.numsteps);
                            this.mtimeadjust.setValue(mtime * 10);
                            dSS.util.ResetDirtyField(this.mtimeadjust);
                        } else {
                            this.numsteps.markInvalid();
                            this.mtimeadjust.markInvalid();
                        }
                        break;
                    default:
                        return;
                }

                if ((this.loaded.motion_time_blink === true) &&
                    (this.loaded.reverse_time_on_dirchange === true) &&
                    (this.loaded.ftime_endpos_threshold_endsw === true) &&
                    (this.loaded.num_steps_mtime_finetune === true)) {
                    this.fireEvent('formloaded', true);
                }
            },
            this);

        this.on('beforedestroy', function() {
                this.closed = true;
            },
            this);

        this.on('submit', function() {
                if (this.getForm().isValid() !== true) {
                    Ext.Msg.alert(dSS._("Error"),
                                  dSS._("Please correct values in the red " +
                                        "marked fields and try again"));
                    return;
                }

                var save = [];

                if (this.mblink.isDirty()) {
                    save.push({
                        url: '/json/device/setConfig',
                        scope: this,
                        params:
                        {
                            'class': 3,
                            index: 0x11,
                            dsid: this.dsid,
                            value: Math.floor(this.mblink.getValue() / 33)
                        },
                        failureMsg: dSS._("Could not save blinking motion " +
                                          "time settings")
                    });
                }

                if (this.reverse.isDirty()) {
                    save.push({
                        url: '/json/device/setConfig',
                        scope: this,
                        params:
                        {
                            'class': 3,
                            index: 0x36,
                            dsid: this.dsid,
                            value: Math.floor(this.reverse.getValue() / 10)
                        },
                        failureMsg: dSS._("Could not save reverse time " +
                                          "on direction change settings")
                    });
                }

                if (this.followup.isDirty()) {
                    save.push({
                        url: '/json/device/setConfig',
                        scope: this,
                        params:
                        {
                            'class': 3,
                            index: 0x38,
                            dsid: this.dsid,
                            value: Math.floor(this.followup.getValue() / 100)
                        },
                        failureMsg: dSS._("Could not save follow-up time " +
                                          "at end position settings")
                    });
                }

                if (this.threshold.isDirty()) {
                    save.push({
                        url: '/json/device/setConfig',
                        scope: this,
                        params:
                        {
                            'class': 3,
                            index: 0x39,
                            dsid: this.dsid,
                            value: Math.floor(this.threshold.getValue()/58)
                        },
                        failureMsg: dSS._("Could not save threshold of end " +
                                          "switch recognition settings")
                    });
                }

                if (this.numsteps.isDirty()) {
                    save.push({
                        url: '/json/device/setConfig',
                        scope: this,
                        params:
                        {
                            'class': 3,
                            index: 0x3a,
                            dsid: this.dsid,
                            value: this.numsteps.getValue()
                        },
                        failureMsg: dSS._("Could not save number of steps " +
                                          "for fine tuning adjustment " +
                                          "settings")
                    });
                }

                if (this.mtimeadjust.isDirty()) {
                    save.push({
                        url: '/json/device/setConfig',
                        scope: this,
                        params:
                        {
                            'class': 3,
                            index: 0x3b,
                            dsid: this.dsid,
                            value: Math.floor(this.mtimeadjust.getValue() / 10)
                        },
                        failureMsg: dSS._("Could not save motion time for " +
                                          "fine tuning adjustment " +
                                          "settings")
                    });
                }

                this.saveOK = true;
                var call = this.saveData.createDelegate(this, [ save, true ]);
                call();
            },
            this);
    },
    saveDataCallback: function(records, state) {
        var call = this.saveData.createDelegate(this, [ records, state ]);
        call();
    },
    saveData: function(records, ok) {
        if (ok === false) {
            this.saveOK = false;
        }
        if (records.length < 1) {
            if (this.saveOK === false) {
                dSS.MessageBus.fireEvent('statusmessage',
                    dSS._("Error") + ": " +
                    dSS._("Some advanced properties could not be saved"));
            }
            this.fireEvent('formsubmitted');
            return;
        }

        var record = records.shift();

        var cb = function(state) {
            var call = this.saveDataCallback.createDelegate(this,
                                                            [ records, state ]);
            call();
        };
        dSS.util.genericRequest(record, cb.createDelegate(this));
    }

});

Ext.reg('shadepropsadvancedform', dSS.devprops.ShadePropertiesAdvancedForm);

